/*
 * ioGame
 * Copyright (C) 2021 - 2023  渔民小镇 （262610965@qq.com、luoyizhu@gmail.com） . All Rights Reserved.
 * # iohao.com . 渔民小镇
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.iohao.mmo.equip.service;

import com.github.javafaker.Faker;
import com.iohao.mmo.equip.entity.*;
import com.iohao.mmo.equip.utils.RandomUtils;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;
import java.math.BigDecimal;
import java.util.List;
import java.util.Locale;
import java.util.Objects;

/**
 * @author 唐斌
 * @date 2023-07-30
 * @description: 装备属性实现类
 */
@Service
@AllArgsConstructor
public class EquipTemplateService {

    final Faker faker = new Faker(Locale.CHINA);
    final MongoTemplate mongoTemplate;

    public EquipTemplate findById(String id) {
        return  mongoTemplate.findById(id, EquipTemplate.class);
    }
    public EquipTemplate findByItemId(String itemId) {
        return  Objects.requireNonNull(mongoTemplate.findOne(Query.query(new Criteria("itemId").is(itemId)), EquipTemplate.class));
    }

    public void save(EquipTemplate equipTemplate) {
        if(StringUtils.isNotBlank(equipTemplate.getId())){
            mongoTemplate.save(equipTemplate);
        }else {
            //TODO 物品：这里需要调用物品新建方法获得物品的id
            String itemId = faker.random().hex(24);
            equipTemplate.setItemId(itemId);
            mongoTemplate.save(equipTemplate);
        }
    }

    public void delBatch(List<String> idList) {
        Query query = new Query(new Criteria("itemId").in(idList));
        mongoTemplate.remove(query, EquipTemplate.class);
        //TODO 物品：删除物品
    }


    public void initEquipTemplate() {
        // 初始化一些装备配置，后期将移到 excel 中做。
        EquipTemplate equipTemplate = new EquipTemplate();
        equipTemplate.setPosition(1);
        equipTemplate.setLevel(10);
        equipTemplate.setFixedEquipPropertyMin(FixedEquipProperty.builder()
                        .hp(100)
                        .mp(100)
                        .physicsDefense(10)
                        .magicDefense(10)
                        .sealDefense(10)
                .build());
        equipTemplate.setFixedEquipPropertyMax(FixedEquipProperty.builder()
                .hp(300)
                .mp(300)
                .physicsDefense(30)
                .magicDefense(30)
                .sealDefense(30)
                .build());
        equipTemplate.setTotalAttrMin(20);
        equipTemplate.setTotalAttrMax(30);

        //TODO 物品：这里需要调用物品新建方法获得物品的id
        String itemId = faker.random().hex(24);
        equipTemplate.setItemId(itemId);

        mongoTemplate.save(equipTemplate);

        // 初始化一些装备配置，后期将移到 excel 中做。
        EquipTemplate equipTemplate2 = new EquipTemplate();
        equipTemplate2.setPosition(2);
        equipTemplate2.setLevel(8);
        equipTemplate2.setFixedEquipPropertyMin(FixedEquipProperty.builder()
                .physicsAttack(8)
                .magicAttack(8)
                .treatAttack(8)
                .sealAttack(8)
                .anger(8)
                .build());
        equipTemplate2.setFixedEquipPropertyMax(FixedEquipProperty.builder()
                .physicsAttack(24)
                .magicAttack(24)
                .treatAttack(24)
                .sealAttack(24)
                .anger(24)
                .build());
        equipTemplate2.setTotalAttrMin(16);
        equipTemplate2.setTotalAttrMax(24);

        //TODO 物品：这里需要调用物品新建方法获得物品的id
        String itemId2 = faker.random().hex(24);
        equipTemplate2.setItemId(itemId2);
        mongoTemplate.save(equipTemplate2);
    }

    /**
     * 根据装备库列表批量随机新的装备
     * @param templateItemIdList
     * @return
     */
    public List<Equip> randomEquipBatch(List<String> templateItemIdList,long userId){
        List<Equip> equipList = templateItemIdList.stream()
                .map(itemId -> randomEquip(itemId,userId))
                .toList();
        return equipList;
    }

    /**
     * 根据装备库随机一件新的装备
     * @param equipTemplateItemId
     * @return
     */
    public Equip randomEquip(String equipTemplateItemId,long userId){
        return randomEquip(equipTemplateItemId,BigDecimal.ZERO,userId);
    }


    /**
     * 根据装备库随机一件新的装备
     * @param equipTemplateItemId 装备库的itemId
     * @param excellentRate 增加的极品率
     * @return
     */
    public Equip randomEquip(String equipTemplateItemId,BigDecimal excellentRate, long userId){
        if(excellentRate == null){
            excellentRate = BigDecimal.ZERO;
        }
        //取到装备库
        EquipTemplate equipTemplate = findByItemId(equipTemplateItemId);
        //将装备库属性拷贝到装备中
        Equip equip = copyTemplate2Equip(equipTemplate,excellentRate);
        equip.setUserId(userId);
        //新建装备对应的物品类
        //TODO 物品：这里需要调用物品新建方法获得物品的id
        String itemId2 = faker.random().hex(24);
        //对应的物品数据是新建的
        equip.setItemId(itemId2);
        mongoTemplate.save(equip);
        return equip;
    }

    public Equip copyTemplate2Equip(EquipTemplate equipTemplate,BigDecimal excellentRate){
        //是否极品
        boolean excellentFlag = RandomUtils.isExcellent(excellentRate);
        //随机总属性点
        int attrTotal = RandomUtils.randomFromExcellent(equipTemplate.getTotalAttrMin(),equipTemplate.getTotalAttrMax(),excellentFlag);
        return Equip.builder()
                .name("测试装备名称") //TODO 物品：获取物品的名称
                .quality(excellentFlag?2:1) //品质
                .fixedEquipProperty(FixedEquipProperty.randomFixed(equipTemplate.getFixedEquipPropertyMin(),equipTemplate.getFixedEquipPropertyMax())) //装备固定属性
                .attrTotal(attrTotal)
                .undistributedAttr(attrTotal)
                .elseEquipProperty(ElseEquipProperty.resetElseEquipProperty()) //初始化分配的具体属性点
                .position(equipTemplate.getPosition())
                .level(equipTemplate.getLevel())
                .fixedEquipPropertyMin(equipTemplate.getFixedEquipPropertyMin())
                .fixedEquipPropertyMax(equipTemplate.getFixedEquipPropertyMax())
                .totalAttrMin(equipTemplate.getTotalAttrMin())
                .totalAttrMax(equipTemplate.getTotalAttrMax())
                .equipTemplateId(equipTemplate.getItemId())
                .build();
    }
}
